@@ -16,7 +16,7 @@ use std::os::unix::fs::OpenOptionsExt;
 /// functions to access members of that directory.
 ///
 /// The accessor functions will enforce that whatever security properties we
-/// checked on the the directory also apply to all of the members that we access
+/// checked on the directory also apply to all of the members that we access
 /// within the directory.
 ///
 /// ## Limitations
@@ -305,6 +305,7 @@ mod test {
     #![allow(clippy::bool_assert_comparison)]
     #![allow(clippy::clone_on_copy)]
     #![allow(clippy::dbg_macro)]
+    #![allow(clippy::mixed_attributes_style)]
     #![allow(clippy::print_stderr)]
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
