@@ -30,6 +30,7 @@
 #![deny(clippy::ref_option_ref)]
 #![warn(clippy::semicolon_if_nothing_returned)]
 #![warn(clippy::trait_duplication_in_bounds)]
+#![deny(clippy::unchecked_duration_subtraction)]
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
@@ -236,10 +237,7 @@ mod os {
 /// * When we compare members of `handle.metadata()` and `path.metadata()`,
 ///   the comparison will return equal if ~~and only if~~
 ///   the two files are truly the same.
-///   * (We know that the "only if" part is not yet guaranteed;
-///     see "Limitations" on lockfile_has_path.)
-///   * When we do switch to look at `file_index`, we will rely only
-///     on the property that a file cannot change its file_index while it is
+///   * We rely on the property that a file cannot change its file_index while it is
 ///     open.
 /// * Deleting the lock file will actually work, since `fslock` opened it with
 ///   FILE_SHARE_DELETE.
@@ -252,7 +250,7 @@ mod os {
 /// * The same is true even on a remote filesystem.
 /// * If someone with read access to the file - eg the human user - opens it for reading
 ///   without FILE_SHARE options, the algorithm will still work and not fail
-///   with a file sharing violation io rror.
+///   with a file sharing violation io error.
 ///   (Or, every program the user might use to randomly peer at files in arti's
 ///   state directory, including the equivalents of `grep -R` and backup programs,
 ///   will use suitable FILE_SHARE options.)
@@ -265,13 +263,6 @@ mod os {
     use winapi::um::fileapi::{GetFileInformationByHandle, BY_HANDLE_FILE_INFORMATION as Info};
 
     /// Return true if `lf` currently exists with the given `path`, and false otherwise.
-    ///
-    /// ## Limitations
-    ///
-    /// This function only looks at `creation_time`, which is not a perfectly
-    /// reliable check.  It should instead look at `volume_serial_number` and
-    /// `file_index`, but those are not yet stable in Rust.
-    ///
     pub(crate) fn lockfile_has_path(lf: &fslock::LockFile, path: &Path) -> std::io::Result<bool> {
         let mut m1: MaybeUninit<Info> = MaybeUninit::uninit();
         let mut m2: MaybeUninit<Info> = MaybeUninit::uninit();
@@ -300,3 +291,45 @@ mod os {
             && i1.dwVolumeSerialNumber == i2.dwVolumeSerialNumber)
     }
 }
+
+#[cfg(test)]
+mod tests {
+    // @@ begin test lint list maintained by maint/add_warning @@
+    #![allow(clippy::bool_assert_comparison)]
+    #![allow(clippy::clone_on_copy)]
+    #![allow(clippy::dbg_macro)]
+    #![allow(clippy::mixed_attributes_style)]
+    #![allow(clippy::print_stderr)]
+    #![allow(clippy::print_stdout)]
+    #![allow(clippy::single_char_pattern)]
+    #![allow(clippy::unwrap_used)]
+    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::useless_vec)]
+    #![allow(clippy::needless_pass_by_value)]
+    //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
+
+    use crate::LockFileGuard;
+    use test_temp_dir::test_temp_dir;
+
+    #[test]
+    fn keep_lock_file_after_drop() {
+        test_temp_dir!().used_by(|dir| {
+            let file = dir.join("file");
+            let flock_guard = LockFileGuard::lock(&file).unwrap();
+            assert!(file.exists());
+            drop(flock_guard);
+            assert!(file.exists());
+        });
+    }
+
+    #[test]
+    fn delete_lock_file_if_requested() {
+        test_temp_dir!().used_by(|dir| {
+            let file = dir.join("file");
+            let flock_guard = LockFileGuard::lock(&file).unwrap();
+            assert!(file.exists());
+            assert!(flock_guard.delete_lock_file(&file).is_ok());
+            assert!(!file.exists());
+        });
+    }
+}
