@@ -41,8 +41,12 @@
 pub(crate) mod celltypes;
 pub(crate) mod halfcirc;
 mod halfstream;
+
 #[cfg(feature = "hs-common")]
 pub mod handshake;
+#[cfg(not(feature = "hs-common"))]
+mod handshake;
+
 #[cfg(feature = "send-control-msg")]
 mod msghandler;
 mod path;
@@ -78,7 +82,7 @@ use tor_linkspec::{CircTarget, LinkSpecType, OwnedChanTarget, RelayIdType};
 
 #[cfg(feature = "hs-service")]
 use {
-    crate::circuit::reactor::IncomingStreamRequestContext,
+    crate::circuit::reactor::StreamReqInfo,
     crate::stream::{IncomingCmdChecker, IncomingStream},
 };
 
@@ -94,6 +98,7 @@ use tor_cell::relaycell::StreamId;
 
 use crate::crypto::handshake::ntor::NtorPublicKey;
 pub use path::{Path, PathEntry};
+pub use reactor::syncview::ClientCircSyncView;
 
 /// The size of the buffer for communication between `ClientCirc` and its reactor.
 pub const CIRCUIT_BUFFER_SIZE: usize = 128;
@@ -539,6 +544,7 @@ impl ClientCirc {
         self: &Arc<ClientCirc>,
         allow_commands: &[tor_cell::relaycell::RelayCmd],
         hop_num: HopNum,
+        filter: impl crate::stream::IncomingStreamRequestFilter,
     ) -> Result<impl futures::Stream<Item = IncomingStream>> {
         use futures::stream::StreamExt;
 
@@ -555,6 +561,7 @@ impl ClientCirc {
                 incoming_sender,
                 hop_num,
                 done: tx,
+                filter: Box::new(filter),
             })
             .map_err(|_| Error::CircuitClosed)?;
 
@@ -565,7 +572,7 @@ impl ClientCirc {
 
         let circ = Arc::clone(self);
         Ok(incoming_receiver.map(move |req_ctx| {
-            let IncomingStreamRequestContext {
+            let StreamReqInfo {
                 req,
                 stream_id,
                 hop_num,
@@ -707,10 +714,14 @@ impl ClientCirc {
     ) -> Result<()> {
         use self::handshake::BoxedClientLayer;
 
+        let protocol = handshake::RelayCryptLayerProtocol::from(protocol);
+        let relay_cell_format = protocol.relay_cell_format();
+
         let BoxedClientLayer { fwd, back, binding } = protocol.construct_layers(role, seed)?;
 
         let (tx, rx) = oneshot::channel();
         let message = CtrlMsg::ExtendVirtual {
+            relay_cell_format,
             cell_crypto: (fwd, back, binding),
             params,
             done: tx,
@@ -1293,6 +1304,7 @@ mod test {
     #![allow(clippy::bool_assert_comparison)]
     #![allow(clippy::clone_on_copy)]
     #![allow(clippy::dbg_macro)]
+    #![allow(clippy::mixed_attributes_style)]
     #![allow(clippy::print_stderr)]
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
@@ -1308,6 +1320,8 @@ mod test {
     use crate::crypto::cell::RelayCellBody;
     #[cfg(feature = "ntor_v3")]
     use crate::crypto::handshake::ntor_v3::NtorV3Server;
+    #[cfg(feature = "hs-service")]
+    use crate::stream::IncomingStreamRequestFilter;
     use chanmsg::{AnyChanMsg, Created2, CreatedFast};
     use futures::channel::mpsc::{Receiver, Sender};
     use futures::io::{AsyncReadExt, AsyncWriteExt};
@@ -1319,7 +1333,7 @@ mod test {
     use tor_basic_utils::test_rng::testing_rng;
     use tor_cell::chancell::{msg as chanmsg, AnyChanCell, BoxedCellBody};
     use tor_cell::relaycell::extend::NtorV3Extension;
-    use tor_cell::relaycell::{msg as relaymsg, AnyRelayMsgOuter, StreamId};
+    use tor_cell::relaycell::{msg as relaymsg, AnyRelayMsgOuter, RelayCellFormat, StreamId};
     use tor_linkspec::OwnedCircTarget;
     use tor_rtcompat::{Runtime, SleepProvider};
     use tracing::trace;
@@ -1588,11 +1602,14 @@ mod test {
             recvcreated: _,
         } = pending;
 
+        // TODO #1067: Support other formats
+        let relay_cell_format = RelayCellFormat::V0;
         for idx in 0_u8..3 {
             let params = CircParameters::default();
             let (tx, rx) = oneshot::channel();
             circ.control
                 .unbounded_send(CtrlMsg::AddFakeHop {
+                    relay_cell_format,
                     fwd_lasthop: idx == 2,
                     rev_lasthop: idx == u8::from(next_msg_from),
                     params,
@@ -1634,7 +1651,10 @@ mod test {
             let rcvd = rx.next().await.unwrap();
             assert_eq!(rcvd.circid(), CircId::new(128));
             let m = match rcvd.into_circid_and_msg().1 {
-                AnyChanMsg::Relay(r) => AnyRelayMsgOuter::decode(r.into_relay_body()).unwrap(),
+                AnyChanMsg::Relay(r) => {
+                    AnyRelayMsgOuter::decode_singleton(RelayCellFormat::V0, r.into_relay_body())
+                        .unwrap()
+                }
                 _ => panic!(),
             };
             assert!(matches!(m.msg(), AnyRelayMsg::BeginDir(_)));
@@ -1730,7 +1750,10 @@ mod test {
             let (id, chmsg) = rx.next().await.unwrap().into_circid_and_msg();
             assert_eq!(id, CircId::new(128));
             let rmsg = match chmsg {
-                AnyChanMsg::RelayEarly(r) => AnyRelayMsgOuter::decode(r.into_relay_body()).unwrap(),
+                AnyChanMsg::RelayEarly(r) => {
+                    AnyRelayMsgOuter::decode_singleton(RelayCellFormat::V0, r.into_relay_body())
+                        .unwrap()
+                }
                 _ => panic!(),
             };
             let e2 = match rmsg.msg() {
@@ -1921,7 +1944,10 @@ mod test {
                 let (id, chmsg) = rx.next().await.unwrap().into_circid_and_msg();
                 assert_eq!(id, CircId::new(128)); // hardcoded circid.
                 let rmsg = match chmsg {
-                    AnyChanMsg::Relay(r) => AnyRelayMsgOuter::decode(r.into_relay_body()).unwrap(),
+                    AnyChanMsg::Relay(r) => {
+                        AnyRelayMsgOuter::decode_singleton(RelayCellFormat::V0, r.into_relay_body())
+                            .unwrap()
+                    }
                     _ => panic!(),
                 };
                 let (streamid, rmsg) = rmsg.into_streamid_and_msg();
@@ -1935,7 +1961,10 @@ mod test {
                 let (id, chmsg) = rx.next().await.unwrap().into_circid_and_msg();
                 assert_eq!(id, CircId::new(128));
                 let rmsg = match chmsg {
-                    AnyChanMsg::Relay(r) => AnyRelayMsgOuter::decode(r.into_relay_body()).unwrap(),
+                    AnyChanMsg::Relay(r) => {
+                        AnyRelayMsgOuter::decode_singleton(RelayCellFormat::V0, r.into_relay_body())
+                            .unwrap()
+                    }
                     _ => panic!(),
                 };
                 let (streamid_2, rmsg) = rmsg.into_streamid_and_msg();
@@ -2001,7 +2030,10 @@ mod test {
             // Read the begindir cell.
             let (_id, chmsg) = rx.next().await.unwrap().into_circid_and_msg();
             let rmsg = match chmsg {
-                AnyChanMsg::Relay(r) => AnyRelayMsgOuter::decode(r.into_relay_body()).unwrap(),
+                AnyChanMsg::Relay(r) => {
+                    AnyRelayMsgOuter::decode_singleton(RelayCellFormat::V0, r.into_relay_body())
+                        .unwrap()
+                }
                 _ => panic!(),
             };
             let (streamid, rmsg) = rmsg.into_streamid_and_msg();
@@ -2018,7 +2050,10 @@ mod test {
                 assert_eq!(id, CircId::new(128));
 
                 let rmsg = match chmsg {
-                    AnyChanMsg::Relay(r) => AnyRelayMsgOuter::decode(r.into_relay_body()).unwrap(),
+                    AnyChanMsg::Relay(r) => {
+                        AnyRelayMsgOuter::decode_singleton(RelayCellFormat::V0, r.into_relay_body())
+                            .unwrap()
+                    }
                     _ => panic!(),
                 };
                 let (streamid2, rmsg) = rmsg.into_streamid_and_msg();
@@ -2164,6 +2199,19 @@ mod test {
         assert_eq!(p.initial_send_window(), 500);
     }
 
+    #[cfg(feature = "hs-service")]
+    struct AllowAllStreamsFilter;
+    #[cfg(feature = "hs-service")]
+    impl IncomingStreamRequestFilter for AllowAllStreamsFilter {
+        fn disposition(
+            &mut self,
+            _ctx: &crate::stream::IncomingStreamRequestContext<'_>,
+            _circ: &ClientCircSyncView<'_>,
+        ) -> Result<crate::stream::IncomingStreamRequestDisposition> {
+            Ok(crate::stream::IncomingStreamRequestDisposition::Accept)
+        }
+    }
+
     #[test]
     #[cfg(feature = "hs-service")]
     fn allow_stream_requests_twice() {
@@ -2175,6 +2223,7 @@ mod test {
                 .allow_stream_requests(
                     &[tor_cell::relaycell::RelayCmd::BEGIN],
                     circ.last_hop_num().unwrap(),
+                    AllowAllStreamsFilter,
                 )
                 .await
                 .unwrap();
@@ -2183,6 +2232,7 @@ mod test {
                 .allow_stream_requests(
                     &[tor_cell::relaycell::RelayCmd::BEGIN],
                     circ.last_hop_num().unwrap(),
+                    AllowAllStreamsFilter,
                 )
                 .await;
 
@@ -2208,6 +2258,7 @@ mod test {
                 .allow_stream_requests(
                     &[tor_cell::relaycell::RelayCmd::BEGIN],
                     circ.last_hop_num().unwrap(),
+                    AllowAllStreamsFilter,
                 )
                 .await
                 .unwrap();
@@ -2284,6 +2335,7 @@ mod test {
                 .allow_stream_requests(
                     &[tor_cell::relaycell::RelayCmd::BEGIN],
                     circ.last_hop_num().unwrap(),
+                    AllowAllStreamsFilter,
                 )
                 .await
                 .unwrap();
@@ -2369,7 +2421,11 @@ mod test {
 
             // Expect to receive incoming streams from hop EXPECTED_HOP
             let mut incoming = circ
-                .allow_stream_requests(&[tor_cell::relaycell::RelayCmd::BEGIN], EXPECTED_HOP.into())
+                .allow_stream_requests(
+                    &[tor_cell::relaycell::RelayCmd::BEGIN],
+                    EXPECTED_HOP.into(),
+                    AllowAllStreamsFilter,
+                )
                 .await
                 .unwrap();
 
